package com.example.maplibre;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.example.goong_maplibre.R;
import com.example.maplibre.adapter.SuggestionAdapter;
import com.example.maplibre.api.IApiService;
import com.example.maplibre.api.RetrofitInstance;
import com.example.maplibre.listener.OnItemClickListener;
import com.example.maplibre.model.AutoComplete;
import com.example.maplibre.model.Location;
import com.example.maplibre.model.Route;
import com.example.maplibre.response.AutoCompleteResponse;
import com.example.maplibre.response.DirectionResponse;
import com.example.maplibre.response.PlaceDetailResponse;
import com.mapbox.android.core.permissions.PermissionsManager;
import com.viettel.vtmsdk.MapVT;
import com.viettel.vtmsdk.maps.MapView;

import org.maplibre.android.annotations.MarkerOptions;
import org.maplibre.android.camera.CameraPosition;
import org.maplibre.android.camera.CameraUpdateFactory;
import org.maplibre.android.geometry.LatLng;
import org.maplibre.android.geometry.LatLngBounds;
import org.maplibre.android.location.LocationComponent;
import org.maplibre.android.maps.MapLibreMap;
import org.maplibre.android.maps.Style;
import org.maplibre.android.style.layers.FillLayer;
import org.maplibre.android.style.layers.LineLayer;
import org.maplibre.android.style.layers.Property;
import org.maplibre.android.style.layers.PropertyFactory;
import org.maplibre.android.style.sources.GeoJsonSource;
import org.maplibre.geojson.Feature;
import org.maplibre.geojson.LineString;
import org.maplibre.geojson.Point;
import org.maplibre.geojson.Polygon;

import java.util.ArrayList;
import java.util.List;

import androidx.activity.result.ActivityResultCallback;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import io.goong.goong_map_sdk.GoongMapView;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class MainActivity extends AppCompatActivity {
    private GoongMapView mapView;
    private MapLibreMap mapboxMap;
    private RecyclerView recyclerView;

    private Button directionButton;

    private SuggestionAdapter adapter;
    private List<AutoComplete> allSuggestions;

    private LatLng selectedLocation;

    private String symbolIconId = "symbolIconId";

    private String geojsonSourceLayerId = "geojsonSourceLayerId";

    private PermissionsManager permissionsManager;
    private LocationComponent locationComponent;

    private final ActivityResultLauncher<String> activityResultLauncher =
            registerForActivityResult(new ActivityResultContracts.RequestPermission(),
                                      new ActivityResultCallback<Boolean>() {
                                          @Override
                                          public void onActivityResult(Boolean result) {
                                              if (result) {
                                                  Toast.makeText(MainActivity.this, "Permission granted!",
                                                                 Toast.LENGTH_SHORT).show();
                                              }
                                          }
                                      });

    public void onMapReady(@NonNull MapLibreMap mapLibreMap) {
        String uri = getResources().getString(R.string.goong_map_url) + "/assets/goong_map_web.json?api_key="
                + getResources().getString(R.string.goong_map_key);
        mapLibreMap.setStyle(
                uri,
                style -> {
                    LatLng start = new LatLng(21.029579719995272, 105.85242472181584);
                    LatLng end = new LatLng(20.9409074, 106.2832288);
                    //Chỉ được sử dụng ảnh dạng bitmap, k phải ảnh vector trong TH custom marker
                    //                        IconFactory iconFactory = IconFactory.getInstance(MainActivity.this);
                    //                        Icon icon = iconFactory.fromResource(R.drawable.blue_marker_view);
                    mapLibreMap.addMarker(new MarkerOptions()
                                                  .position(start));

                    mapLibreMap.addMarker(new MarkerOptions()
                                                  .position(end)
                                         );
                    CameraPosition position = new CameraPosition.Builder()
                            .target(start)
                            .zoom(15)
                            .tilt(20)
                            .build();
                    mapLibreMap.animateCamera(CameraUpdateFactory.newCameraPosition(position), 1200);
                });
        this.mapboxMap = mapLibreMap;
    }

    private void initViettelMap() {
        MapView vtMapView = findViewById(R.id.mapViewViettel);
        vtMapView.getMapAsync(vtMap -> {
            String uri = getResources().getString(R.string.goong_map_url) + "/assets/goong_map_web.json?api_key="
                    + getResources().getString(R.string.goong_map_key);
            vtMap.setStyle(new com.viettel.vtmsdk.maps.Style.Builder().fromUri(uri), style -> {
                vtMap.setCameraPosition(new com.viettel.vtmsdk.camera.CameraPosition.Builder()
                                                .target(new com.viettel.vtmsdk.geometry.LatLng(21.0000, 105.8500))
                                                .zoom(16)
                                                .build());
            });
        });
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        MapVT.getInstance(this,
                getResources().getString(R.string.vt_mapbox_access_token));
        setContentView(R.layout.activity_main);
        initViettelMap();
        mapView = findViewById(R.id.mapView);
        mapView.onCreate(savedInstanceState);
        mapView.getMapAsync(this::onMapReady);
        directionButton = findViewById(R.id.directionBtn);
        directionButton.setVisibility(View.GONE);
        EditText searchEditText = findViewById(R.id.search_edit_text);
        recyclerView = findViewById(R.id.recycler_view);
        recyclerView.setVisibility(View.VISIBLE);
        allSuggestions = new ArrayList<>();
        //Gán sự kiện khi click vào từng item
        this.adapter = new SuggestionAdapter(allSuggestions, new OnItemClickListener() {
            @Override
            public void onItemClick(int position, String placeId) {
                //TODO: Chuyển camera đến điểm đó và gán marker
                //Và chỉ đường từ điểm đó đến điểm mình search
                fetchDetailLocation(placeId);
                recyclerView.setVisibility(View.GONE);
                directionButton.setVisibility(View.VISIBLE);
            }
        });
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setAdapter(adapter);

        searchEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                search(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        searchEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    // Perform actions when EditText loses focus (blurred)
                    // For example, hide the keyboard
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(searchEditText.getWindowToken(), 0);
                    recyclerView.setVisibility(View.GONE);
                } else {
                    // EditText gained focus, show the RecyclerView
                    recyclerView.setVisibility(View.VISIBLE);
                }
            }
        });

        directionButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Style style = mapboxMap.getStyle();
                LatLng start = new LatLng(21.029579719995272, 105.85242472181584);
                fetchDirections(start, selectedLocation, style);
            }
        });
    }

    private void search(String textSearch) {
        if (recyclerView != null) {
            recyclerView.setVisibility(View.VISIBLE);
        }
        List<AutoComplete> filteredList = new ArrayList<>();
        try {
            if (textSearch != null) {
                IApiService service =
                        RetrofitInstance.getRetrofitInstance(getResources().getString(R.string.goong_api_url))
                                        .create(IApiService.class);
                Call<AutoCompleteResponse> call =
                        service.getAutoComplete(textSearch, getResources().getString(R.string.goong_api_key));
                call.enqueue(new Callback<AutoCompleteResponse>() {
                    @Override
                    public void onResponse(Call<AutoCompleteResponse> call, Response<AutoCompleteResponse> response) {
                        if (response != null && response.isSuccessful()) {
                            AutoCompleteResponse data = response.body();
                            if (data != null && data.getPredictions() != null) {
                                List<AutoComplete> autoCompletes = data.getPredictions();
                                for (AutoComplete item : autoCompletes) {
                                    filteredList.add(item);
                                }
                                if (filteredList != null && MainActivity.this.adapter != null) {
                                    MainActivity.this.adapter.updateSuggestions(filteredList);
                                }
                            }
                        } else {
                            // Handle error
                        }
                    }

                    @Override
                    public void onFailure(Call<AutoCompleteResponse> call, Throwable t) {
                        Toast.makeText(MainActivity.this, "Auto complete is error!!!", Toast.LENGTH_SHORT).show();
                    }
                });
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        if (filteredList != null && this.adapter != null) {
            this.adapter.updateSuggestions(filteredList);
        }
    }

    private void fetchDetailLocation(String placeId) {
        Style style = mapboxMap.getStyle();
        IApiService service = RetrofitInstance.getRetrofitInstance(getResources().getString(R.string.goong_api_url))
                                              .create(IApiService.class);
        Call<PlaceDetailResponse> call =
                service.getPlaceDetail(placeId, getResources().getString(R.string.goong_api_key));
        call.enqueue(new Callback<PlaceDetailResponse>() {
            @Override
            public void onResponse(Call<PlaceDetailResponse> call, Response<PlaceDetailResponse> response) {
                //TODO: Hiển thị đánh marker cho điểm theo tọa độ
                if (response != null && response.isSuccessful()) {
                    PlaceDetailResponse data = response.body();
                    if (data != null && data.getResult() != null && data.getResult().getGeometry() != null) {
                        Location location = data.getResult().getGeometry().getLocation();
                        try {
                            LatLng center = new LatLng(Double.parseDouble(location.getLat()),
                                                       Double.parseDouble(location.getLng()));
                            drawCircleLineAndMarker(center, style);
                            selectedLocation = center;
                            mapboxMap.clear();
                            mapboxMap.addMarker(new MarkerOptions()
                                                        .position(center));
                        } catch (Exception e) {
                            Toast.makeText(MainActivity.this, "Parsing is error!!!", Toast.LENGTH_SHORT).show();
                        }
                    }
                }
            }

            @Override
            public void onFailure(Call<PlaceDetailResponse> call, Throwable t) {
                Toast.makeText(MainActivity.this, "FetchDetailLocation is error!!!", Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void fetchDirections(LatLng start, LatLng end, Style style) {
        String origin = start.getLatitude() + "," + start.getLongitude(),
                destination = end.getLatitude() + "," + end.getLongitude();
        IApiService service = RetrofitInstance.getRetrofitInstance(getResources().getString(R.string.goong_api_url))
                                              .create(IApiService.class);
        Call<DirectionResponse> call =
                service.getDirection(origin, destination, "car", getResources().getString(R.string.goong_api_key));
        call.enqueue(new Callback<DirectionResponse>() {
            @Override
            public void onResponse(Call<DirectionResponse> call, Response<DirectionResponse> response) {
                //Gọi API lấy danh sách các điểm
                if (response != null && response.isSuccessful()) {
                    DirectionResponse data = response.body();
                    if (data != null && data.getRoutes() != null) {
                        Route route = data.getRoutes().get(0);
                        if (route.getOverviewPolyline() != null && route.getOverviewPolyline() != null &&
                                route.getOverviewPolyline().getPoints() != null) {
                            String geometry = route.getOverviewPolyline().getPoints();
                            List<Point> points = LineString.fromPolyline(geometry, 5).coordinates();
                            drawLineBetweenTwoPoint(points, style);
                            mapboxMap.clear();
                            mapboxMap.addMarker(new MarkerOptions()
                                                        .position(start));
                            mapboxMap.addMarker(new MarkerOptions()
                                                        .position(end));
                            bound(start, end);
                        }
                    }
                }
            }

            @Override
            public void onFailure(Call<DirectionResponse> call, Throwable t) {
                Log.d("Call error", "Call error");
            }
        });
    }

    public void bound(LatLng start, LatLng end) {
        LatLngBounds latLngBounds = new LatLngBounds.Builder()
                .include(start) // Northeast
                .include(end) // Southwest
                .build();

        mapboxMap.easeCamera(CameraUpdateFactory.newLatLngBounds(latLngBounds, 20), 5000);
    }

    public void drawLineBetweenTwoPoint(List<Point> points, Style style) {
        LineString lineString = LineString.fromLngLats(points);
        Feature feature = Feature.fromGeometry(lineString);
        GeoJsonSource geoJsonSource = new GeoJsonSource("line-source", feature);
        style.addSource(geoJsonSource);
        style.addLayer(new LineLayer("linelayer", "line-source")
                               .withProperties(PropertyFactory.lineCap(Property.LINE_CAP_SQUARE),
                                               PropertyFactory.lineJoin(Property.LINE_JOIN_MITER),
                                               PropertyFactory.lineOpacity(.7f),
                                               PropertyFactory.lineWidth(7f),
                                               PropertyFactory.lineColor(Color.parseColor("#3887be"))));
    }

    public void drawCircleLineAndMarker(LatLng center, Style style) {
        List<Point> points = this.getCircleLatLng(center, 300);
        //Vẽ fill color hình tròn
        Feature circlePolygon = Feature.fromGeometry(createPolygonFromPoints(points));
        style.addSource(new GeoJsonSource("circle_polygon", circlePolygon));
        style.addLayer(new FillLayer("polygon-layer", "circle_polygon")
                               .withProperties(
                                       PropertyFactory.fillOpacity(0.2f),
                                       PropertyFactory.fillColor(Color.parseColor("#588888"))
                                              ));
        //Vẽ line xung quanh
        style.addLayer(new LineLayer("circle_layer", "circle_source")
                               .withProperties(PropertyFactory.lineCap(Property.LINE_CAP_SQUARE),
                                               PropertyFactory.lineJoin(Property.LINE_JOIN_MITER),
                                               PropertyFactory.lineOpacity(.7f),
                                               PropertyFactory.lineWidth(0.3f),
                                               PropertyFactory.lineColor(Color.parseColor("#588888"))));

        CameraPosition position = new CameraPosition.Builder()
                .target(center)
                .zoom(15)
                .tilt(20)
                .build();
        mapboxMap.animateCamera(CameraUpdateFactory.newCameraPosition(position), 1200);

    }

    public static Polygon createPolygonFromPoints(List<Point> points) {
        List<List<Point>> polygonCoordinates = new ArrayList<>();
        polygonCoordinates.add(points);
        return Polygon.fromLngLats(polygonCoordinates);
    }

    public List<Point> getCircleLatLng(LatLng center, double radius) {
        List<Point> result = new ArrayList<>();
        int points = 64;
        double[][] coordinates = new double[points + 1][2];
        for (int i = 0; i < points; i++) {
            double angle = i * (2 * Math.PI / points);
            double dx = radius * Math.cos(angle);
            double dy = radius * Math.sin(angle);
            coordinates[i] = new double[] {
                    center.getLongitude() + (dx / 110540f),
                    center.getLatitude() + (dy / 110540f)
            };
        }
        coordinates[points] = coordinates[0]; // Close the circle
        for (double[] coordinate : coordinates) {
            result.add(Point.fromLngLat(coordinate[0], coordinate[1]));
        }
        return result;
    }

    @Override
    protected void onStart() {
        super.onStart();
        mapView.onStart();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mapView.onStop();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        mapView.onSaveInstanceState(outState);
    }

    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mapView.onLowMemory();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mapView.onDestroy();
    }
}
